/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef ZzVerbose_h
#include "ZzVerbose.h"
#endif //ZzVerbose_h

#import "MILock.h"

@implementation MILock


- (id)init
{
	self = [super init];
	fReadWriteLock = [[NSLock alloc] init];
	fAccessLock = [[NSLock alloc] init];
	fReadLockCount = 0;
	fLockType = MINoLock;
	
	return self;
}

- (void)lockRead
{
	[fAccessLock lock];
	
	fReadLockCount++;
	
	if ( fReadLockCount == 1 )
	{
		[fReadWriteLock lock];
	}
	
	fLockType = MIReadLockType;
	
	[fAccessLock unlock];
}

- (void)unlockRead
{
	[fAccessLock lock];
	
	fReadLockCount--;
	
	if ( fReadLockCount < 0 )
	{
		verntl(interfaceStephan,0,"ERROR!!!\n\nreadLockCount negative!");
		fReadLockCount = 0;
	}
	
	if ( !fReadLockCount )
	{
		[fReadWriteLock unlock];
		fLockType = MINoLock;
	}

	[fAccessLock unlock];
}

- (void)lockWrite
{
	[fAccessLock lock];

	[fReadWriteLock lock];
	fLockType = MIWriteLockType;
	
	[fAccessLock unlock];
}

- (void)unlockWrite
{
	[fAccessLock lock];
	
	[fReadWriteLock unlock];
	fLockType = MINoLock;
	[fAccessLock unlock];
}

@end
